@@ -24,7 +24,7 @@
 #include "wine/debug.h"
 #include "wine/strmbase.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(amstream);
+WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
 static const WCHAR sink_id[] = L"I{A35FF56A-9FDA-11D0-8FDF-00C04FD9189D}";
 
@@ -77,6 +77,7 @@ struct ddraw_sample
     STREAM_TIME end_time;
     BOOL continuous_update;
     CONDITION_VARIABLE update_cv;
+    HANDLE external_event;
 
     struct list entry;
     HRESULT update_hr;
@@ -91,6 +92,8 @@ static void remove_queued_update(struct ddraw_sample *sample)
     sample->busy = FALSE;
     list_remove(&sample->entry);
     WakeConditionVariable(&sample->update_cv);
+    if (sample->external_event)
+        SetEvent(sample->external_event);
 }
 
 static void flush_update_queue(struct ddraw_stream *stream, HRESULT update_hr)
@@ -1177,23 +1180,36 @@ static HRESULT WINAPI ddraw_sink_EndOfStream(IPin *iface)
 
     LeaveCriticalSection(&stream->cs);
 
+    /* Calling IMediaStreamFilter::EndOfStream() inside the critical section
+     * would invert the locking order, so we must leave it first to avoid
+     * the streaming thread deadlocking on the filter's critical section. */
+    IMediaStreamFilter_EndOfStream(stream->filter);
+
     return S_OK;
 }
 
 static HRESULT WINAPI ddraw_sink_BeginFlush(IPin *iface)
 {
     struct ddraw_stream *stream = impl_from_IPin(iface);
+    BOOL cancel_eos;
 
     TRACE("stream %p.\n", stream);
 
     EnterCriticalSection(&stream->cs);
 
+    cancel_eos = stream->eos;
+
     stream->flushing = TRUE;
     stream->eos = FALSE;
     WakeConditionVariable(&stream->update_queued_cv);
 
     LeaveCriticalSection(&stream->cs);
 
+    /* Calling IMediaStreamFilter::Flush() inside the critical section would
+     * invert the locking order, so we must leave it first to avoid the
+     * application thread deadlocking on the filter's critical section. */
+    IMediaStreamFilter_Flush(stream->filter, cancel_eos);
+
     return S_OK;
 }
 
@@ -1583,12 +1599,6 @@ static HRESULT WINAPI ddraw_sample_Update(IDirectDrawStreamSample *iface,
         return E_NOTIMPL;
     }
 
-    if (event)
-    {
-        FIXME("Event parameter support is not implemented!\n");
-        return E_NOTIMPL;
-    }
-
     EnterCriticalSection(&sample->parent->cs);
 
     if (sample->parent->state != State_Running)
@@ -1612,10 +1622,11 @@ static HRESULT WINAPI ddraw_sample_Update(IDirectDrawStreamSample *iface,
 
     sample->update_hr = MS_S_NOUPDATE;
     sample->busy = TRUE;
+    sample->external_event = event;
     list_add_tail(&sample->parent->update_queue, &sample->entry);
     WakeConditionVariable(&sample->parent->update_queued_cv);
 
-    if (flags & SSUPDATE_ASYNC)
+    if ((flags & SSUPDATE_ASYNC) || event)
     {
         LeaveCriticalSection(&sample->parent->cs);
         return MS_S_PENDING;
