@@ -140,10 +140,13 @@ static HRESULT mousedev_enum_device(DWORD dwDevType, DWORD dwFlags, LPDIDEVICEIN
 
 static HRESULT alloc_device( REFGUID rguid, IDirectInputImpl *dinput, SysMouseImpl **out )
 {
+    static const WCHAR mouse_wrap_override_w[] = {'M','o','u','s','e','W','a','r','p','O','v','e','r','r','i','d','e',0};
+    static const WCHAR disable_w[] = {'d','i','s','a','b','l','e',0};
+    static const WCHAR force_w[] = {'f','o','r','c','e',0};
     SysMouseImpl* newDevice;
     LPDIDATAFORMAT df = NULL;
     unsigned i;
-    char buffer[20];
+    WCHAR buffer[20];
     HKEY hkey, appkey;
     HRESULT hr;
 
@@ -155,11 +158,11 @@ static HRESULT alloc_device( REFGUID rguid, IDirectInputImpl *dinput, SysMouseIm
     newDevice->base.dwCoopLevel = DISCL_NONEXCLUSIVE | DISCL_BACKGROUND;
 
     get_app_key(&hkey, &appkey);
-    if (!get_config_key(hkey, appkey, "MouseWarpOverride", buffer, sizeof(buffer)))
+    if (!get_config_key(hkey, appkey, mouse_wrap_override_w, buffer, sizeof(buffer)))
     {
-        if (!_strnicmp(buffer, "disable", -1))
+        if (!strncmpiW(buffer, disable_w, -1))
             newDevice->warp_override = WARP_DISABLE;
-        else if (!_strnicmp(buffer, "force", -1))
+        else if (!strncmpiW(buffer, force_w, -1))
             newDevice->warp_override = WARP_FORCE_ON;
     }
     if (appkey) RegCloseKey(appkey);
@@ -231,6 +234,7 @@ void dinput_mouse_rawinput_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPA
     POINT rel, pt;
     DWORD seq;
     int i, wdata = 0;
+    BOOL notify = FALSE;
 
     static const USHORT mouse_button_flags[] =
     {
@@ -274,12 +278,18 @@ void dinput_mouse_rawinput_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPA
     }
 
     if (rel.x)
+    {
         queue_event( iface, DIDFT_MAKEINSTANCE(WINE_MOUSE_X_AXIS_INSTANCE) | DIDFT_RELAXIS,
                      pt.x, GetCurrentTime(), seq );
+        notify = TRUE;
+    }
 
     if (rel.y)
+    {
         queue_event( iface, DIDFT_MAKEINSTANCE(WINE_MOUSE_Y_AXIS_INSTANCE) | DIDFT_RELAXIS,
                      pt.y, GetCurrentTime(), seq );
+        notify = TRUE;
+    }
 
     if (rel.x || rel.y)
     {
@@ -293,6 +303,7 @@ void dinput_mouse_rawinput_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPA
         This->m_state.lZ += (wdata = (SHORT)ri->data.mouse.usButtonData);
         queue_event( iface, DIDFT_MAKEINSTANCE(WINE_MOUSE_Z_AXIS_INSTANCE) | DIDFT_RELAXIS,
                      wdata, GetCurrentTime(), seq );
+        notify = TRUE;
     }
 
     for (i = 0; i < ARRAY_SIZE(mouse_button_flags); ++i)
@@ -302,9 +313,11 @@ void dinput_mouse_rawinput_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPA
             This->m_state.rgbButtons[i / 2] = 0x80 - (i % 2) * 0x80;
             queue_event( iface, DIDFT_MAKEINSTANCE(WINE_MOUSE_BUTTONS_INSTANCE +(i / 2) ) | DIDFT_PSHBUTTON,
                          This->m_state.rgbButtons[i / 2], GetCurrentTime(), seq );
+            notify = TRUE;
         }
     }
 
+    if (notify && This->base.hEvent) SetEvent( This->base.hEvent );
     LeaveCriticalSection( &This->base.crit );
 }
 
@@ -314,6 +327,7 @@ int dinput_mouse_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam
     MSLLHOOKSTRUCT *hook = (MSLLHOOKSTRUCT *)lparam;
     SysMouseImpl *This = impl_from_IDirectInputDevice8W( iface );
     int wdata = 0, inst_id = -1, ret = 0;
+    BOOL notify = FALSE;
 
     TRACE("msg %lx @ (%d %d)\n", wparam, hook->pt.x, hook->pt.y);
 
@@ -344,8 +358,11 @@ int dinput_mouse_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam
             {
                 /* Already have X, need to queue it */
                 if (inst_id != -1)
+                {
                     queue_event(iface, inst_id,
                                 wdata, GetCurrentTime(), This->base.dinput->evsequence);
+                    notify = TRUE;
+                }
                 inst_id = DIDFT_MAKEINSTANCE(WINE_MOUSE_Y_AXIS_INSTANCE) | DIDFT_RELAXIS;
                 wdata = pt1.y;
             }
@@ -405,8 +422,10 @@ int dinput_mouse_hook( IDirectInputDevice8W *iface, WPARAM wparam, LPARAM lparam
         _dump_mouse_state(&This->m_state);
         queue_event(iface, inst_id,
                     wdata, GetCurrentTime(), This->base.dinput->evsequence++);
+        notify = TRUE;
     }
 
+    if (notify && This->base.hEvent) SetEvent( This->base.hEvent );
     LeaveCriticalSection(&This->base.crit);
     return ret;
 }
