@@ -187,6 +187,8 @@ static const MSSTYLES_PROPERTY_MAP mapProperty[] = {
     {TMT_INT, TMT_MINDPI3, L"MINDPI3"},
     {TMT_INT, TMT_MINDPI4, L"MINDPI4"},
     {TMT_INT, TMT_MINDPI5, L"MINDPI5"},
+    {TMT_INT, TMT_MINDPI6, L"MINDPI6"},
+    {TMT_INT, TMT_MINDPI7, L"MINDPI7"},
     {TMT_FONT, TMT_GLYPHFONT, L"GLYPHFONT"},
     {TMT_FILENAME, TMT_IMAGEFILE, L"IMAGEFILE"},
     {TMT_FILENAME, TMT_IMAGEFILE1, L"IMAGEFILE1"},
@@ -194,6 +196,8 @@ static const MSSTYLES_PROPERTY_MAP mapProperty[] = {
     {TMT_FILENAME, TMT_IMAGEFILE3, L"IMAGEFILE3"},
     {TMT_FILENAME, TMT_IMAGEFILE4, L"IMAGEFILE4"},
     {TMT_FILENAME, TMT_IMAGEFILE5, L"IMAGEFILE5"},
+    {TMT_FILENAME, TMT_IMAGEFILE6, L"IMAGEFILE6"},
+    {TMT_FILENAME, TMT_IMAGEFILE7, L"IMAGEFILE7"},
     {TMT_FILENAME, TMT_STOCKIMAGEFILE, L"STOCKIMAGEFILE"},
     {TMT_FILENAME, TMT_GLYPHIMAGEFILE, L"GLYPHIMAGEFILE"},
     {TMT_STRING, TMT_TEXT, L"TEXT"},
@@ -206,6 +210,8 @@ static const MSSTYLES_PROPERTY_MAP mapProperty[] = {
     {TMT_POSITION, TMT_MINSIZE4, L"MINSIZE4"},
     {TMT_POSITION, TMT_MINSIZE5, L"MINSIZE5"},
     {TMT_POSITION, TMT_NORMALSIZE, L"NORMALSIZE"},
+    {TMT_POSITION, TMT_MINSIZE6, L"MINSIZE6"},
+    {TMT_POSITION, TMT_MINSIZE7, L"MINSIZE7"},
     {TMT_MARGINS, TMT_SIZINGMARGINS, L"SIZINGMARGINS"},
     {TMT_MARGINS, TMT_CONTENTMARGINS, L"CONTENTMARGINS"},
     {TMT_MARGINS, TMT_CAPTIONMARGINS, L"CAPTIONMARGINS"},
@@ -361,21 +367,21 @@ static const MSSTYLES_CLASS_MAP classButton[] = {
     {BP_RADIOBUTTON, RBS_CHECKEDPRESSED, L"CHECKEDPRESSED"},
     {BP_RADIOBUTTON, RBS_CHECKEDDISABLED, L"CHECKEDDISABLED"},
     {BP_CHECKBOX, 0, L"CHECKBOX"},
-    {BP_RADIOBUTTON, CBS_UNCHECKEDNORMAL, L"UNCHECKEDNORMAL"},
-    {BP_RADIOBUTTON, CBS_UNCHECKEDHOT, L"UNCHECKEDHOT"},
-    {BP_RADIOBUTTON, CBS_UNCHECKEDPRESSED, L"UNCHECKEDPRESSED"},
-    {BP_RADIOBUTTON, CBS_UNCHECKEDDISABLED, L"UNCHECKEDDISABLED"},
-    {BP_RADIOBUTTON, CBS_CHECKEDNORMAL, L"CHECKEDNORMAL"},
-    {BP_RADIOBUTTON, CBS_CHECKEDHOT, L"CHECKEDHOT"},
-    {BP_RADIOBUTTON, CBS_CHECKEDPRESSED, L"CHECKEDPRESSED"},
-    {BP_RADIOBUTTON, CBS_CHECKEDDISABLED, L"CHECKEDDISABLED"},
-    {BP_RADIOBUTTON, CBS_MIXEDNORMAL, L"MIXEDNORMAL"},
-    {BP_RADIOBUTTON, CBS_MIXEDHOT, L"MIXEDHOT"},
-    {BP_RADIOBUTTON, CBS_MIXEDPRESSED, L"MIXEDPRESSED"},
-    {BP_RADIOBUTTON, CBS_MIXEDDISABLED, L"MIXEDDISABLED"},
+    {BP_CHECKBOX, CBS_UNCHECKEDNORMAL, L"UNCHECKEDNORMAL"},
+    {BP_CHECKBOX, CBS_UNCHECKEDHOT, L"UNCHECKEDHOT"},
+    {BP_CHECKBOX, CBS_UNCHECKEDPRESSED, L"UNCHECKEDPRESSED"},
+    {BP_CHECKBOX, CBS_UNCHECKEDDISABLED, L"UNCHECKEDDISABLED"},
+    {BP_CHECKBOX, CBS_CHECKEDNORMAL, L"CHECKEDNORMAL"},
+    {BP_CHECKBOX, CBS_CHECKEDHOT, L"CHECKEDHOT"},
+    {BP_CHECKBOX, CBS_CHECKEDPRESSED, L"CHECKEDPRESSED"},
+    {BP_CHECKBOX, CBS_CHECKEDDISABLED, L"CHECKEDDISABLED"},
+    {BP_CHECKBOX, CBS_MIXEDNORMAL, L"MIXEDNORMAL"},
+    {BP_CHECKBOX, CBS_MIXEDHOT, L"MIXEDHOT"},
+    {BP_CHECKBOX, CBS_MIXEDPRESSED, L"MIXEDPRESSED"},
+    {BP_CHECKBOX, CBS_MIXEDDISABLED, L"MIXEDDISABLED"},
     {BP_GROUPBOX, 0, L"GROUPBOX"},
-    {BP_RADIOBUTTON, GBS_NORMAL, L"NORMAL"},
-    {BP_RADIOBUTTON, GBS_DISABLED, L"DISABLED"},
+    {BP_GROUPBOX, GBS_NORMAL, L"NORMAL"},
+    {BP_GROUPBOX, GBS_DISABLED, L"DISABLED"},
     {BP_USERBUTTON, 0, L"USERBUTTON"},
     {BP_COMMANDLINK, 0, L"COMMANDLINK"},
     {BP_COMMANDLINK, CMDLS_NORMAL, L"NORMAL"},
