@@ -55,6 +55,8 @@ static WCHAR szCurrentTheme[MAX_PATH];
 static WCHAR szCurrentColor[64];
 static WCHAR szCurrentSize[64];
 
+struct user_api_hook user_api = {0};
+
 /***********************************************************************/
 
 static BOOL CALLBACK UXTHEME_broadcast_msg_enumchild (HWND hWnd, LPARAM msg)
@@ -197,39 +199,39 @@ static void UXTHEME_LoadTheme(void)
 
 /***********************************************************************/
 
-static const char * const SysColorsNames[] =
+static const WCHAR * const SysColorsNames[] =
 {
-    "Scrollbar",                /* COLOR_SCROLLBAR */
-    "Background",               /* COLOR_BACKGROUND */
-    "ActiveTitle",              /* COLOR_ACTIVECAPTION */
-    "InactiveTitle",            /* COLOR_INACTIVECAPTION */
-    "Menu",                     /* COLOR_MENU */
-    "Window",                   /* COLOR_WINDOW */
-    "WindowFrame",              /* COLOR_WINDOWFRAME */
-    "MenuText",                 /* COLOR_MENUTEXT */
-    "WindowText",               /* COLOR_WINDOWTEXT */
-    "TitleText",                /* COLOR_CAPTIONTEXT */
-    "ActiveBorder",             /* COLOR_ACTIVEBORDER */
-    "InactiveBorder",           /* COLOR_INACTIVEBORDER */
-    "AppWorkSpace",             /* COLOR_APPWORKSPACE */
-    "Hilight",                  /* COLOR_HIGHLIGHT */
-    "HilightText",              /* COLOR_HIGHLIGHTTEXT */
-    "ButtonFace",               /* COLOR_BTNFACE */
-    "ButtonShadow",             /* COLOR_BTNSHADOW */
-    "GrayText",                 /* COLOR_GRAYTEXT */
-    "ButtonText",               /* COLOR_BTNTEXT */
-    "InactiveTitleText",        /* COLOR_INACTIVECAPTIONTEXT */
-    "ButtonHilight",            /* COLOR_BTNHIGHLIGHT */
-    "ButtonDkShadow",           /* COLOR_3DDKSHADOW */
-    "ButtonLight",              /* COLOR_3DLIGHT */
-    "InfoText",                 /* COLOR_INFOTEXT */
-    "InfoWindow",               /* COLOR_INFOBK */
-    "ButtonAlternateFace",      /* COLOR_ALTERNATEBTNFACE */
-    "HotTrackingColor",         /* COLOR_HOTLIGHT */
-    "GradientActiveTitle",      /* COLOR_GRADIENTACTIVECAPTION */
-    "GradientInactiveTitle",    /* COLOR_GRADIENTINACTIVECAPTION */
-    "MenuHilight",              /* COLOR_MENUHILIGHT */
-    "MenuBar",                  /* COLOR_MENUBAR */
+    L"Scrollbar",               /* COLOR_SCROLLBAR */
+    L"Background",              /* COLOR_BACKGROUND */
+    L"ActiveTitle",             /* COLOR_ACTIVECAPTION */
+    L"InactiveTitle",           /* COLOR_INACTIVECAPTION */
+    L"Menu",                    /* COLOR_MENU */
+    L"Window",                  /* COLOR_WINDOW */
+    L"WindowFrame",             /* COLOR_WINDOWFRAME */
+    L"MenuText",                /* COLOR_MENUTEXT */
+    L"WindowText",              /* COLOR_WINDOWTEXT */
+    L"TitleText",               /* COLOR_CAPTIONTEXT */
+    L"ActiveBorder",            /* COLOR_ACTIVEBORDER */
+    L"InactiveBorder",          /* COLOR_INACTIVEBORDER */
+    L"AppWorkSpace",            /* COLOR_APPWORKSPACE */
+    L"Hilight",                 /* COLOR_HIGHLIGHT */
+    L"HilightText",             /* COLOR_HIGHLIGHTTEXT */
+    L"ButtonFace",              /* COLOR_BTNFACE */
+    L"ButtonShadow",            /* COLOR_BTNSHADOW */
+    L"GrayText",                /* COLOR_GRAYTEXT */
+    L"ButtonText",              /* COLOR_BTNTEXT */
+    L"InactiveTitleText",       /* COLOR_INACTIVECAPTIONTEXT */
+    L"ButtonHilight",           /* COLOR_BTNHIGHLIGHT */
+    L"ButtonDkShadow",          /* COLOR_3DDKSHADOW */
+    L"ButtonLight",             /* COLOR_3DLIGHT */
+    L"InfoText",                /* COLOR_INFOTEXT */
+    L"InfoWindow",              /* COLOR_INFOBK */
+    L"ButtonAlternateFace",     /* COLOR_ALTERNATEBTNFACE */
+    L"HotTrackingColor",        /* COLOR_HOTLIGHT */
+    L"GradientActiveTitle",     /* COLOR_GRADIENTACTIVECAPTION */
+    L"GradientInactiveTitle",   /* COLOR_GRADIENTINACTIVECAPTION */
+    L"MenuHilight",             /* COLOR_MENUHILIGHT */
+    L"MenuBar",                 /* COLOR_MENUBAR */
 };
 
 static const WCHAR strColorKey[] = L"Control Panel\\Colors";
@@ -249,9 +251,9 @@ static const struct BackupSysParam
 
 static void save_sys_colors (HKEY baseKey)
 {
-    char colorStr[13];
+    WCHAR colorStr[13];
     HKEY hKey;
-    int i;
+    int i, length;
 
     if (RegCreateKeyExW( baseKey, strColorKey,
                          0, 0, 0, KEY_ALL_ACCESS,
@@ -260,12 +262,11 @@ static void save_sys_colors (HKEY baseKey)
         for (i = 0; i < NUM_SYS_COLORS; i++)
         {
             COLORREF col = GetSysColor (i);
-        
-            sprintf (colorStr, "%d %d %d", 
-                GetRValue (col), GetGValue (col), GetBValue (col));
 
-            RegSetValueExA (hKey, SysColorsNames[i], 0, REG_SZ, 
-                (BYTE*)colorStr, strlen (colorStr)+1);
+            length = swprintf(colorStr, ARRAY_SIZE(colorStr), L"%d %d %d", GetRValue(col),
+                              GetGValue(col), GetBValue(col));
+            RegSetValueExW(hKey, SysColorsNames[i], 0, REG_SZ, (BYTE *)colorStr,
+                           (length + 1) * sizeof(WCHAR));
         }
         RegCloseKey (hKey);
     }
@@ -276,9 +277,12 @@ static void save_sys_colors (HKEY baseKey)
  * is deactivated */
 static void UXTHEME_BackupSystemMetrics(void)
 {
+    DPI_AWARENESS_CONTEXT old_context;
     HKEY hKey;
     const struct BackupSysParam* bsp = backupSysParams;
 
+    old_context = SetThreadDpiAwarenessContext(DPI_AWARENESS_CONTEXT_UNAWARE);
+
     if (RegCreateKeyExW( HKEY_CURRENT_USER, szThemeManager,
                          0, 0, 0, KEY_ALL_ACCESS,
                          0, &hKey, 0) == ERROR_SUCCESS)
@@ -315,14 +319,19 @@ static void UXTHEME_BackupSystemMetrics(void)
     
         RegCloseKey (hKey);
     }
+
+    SetThreadDpiAwarenessContext(old_context);
 }
 
 /* Read back old settings after a theme was deactivated */
 static void UXTHEME_RestoreSystemMetrics(void)
 {
+    DPI_AWARENESS_CONTEXT old_context;
     HKEY hKey;
     const struct BackupSysParam* bsp = backupSysParams;
 
+    old_context = SetThreadDpiAwarenessContext(DPI_AWARENESS_CONTEXT_UNAWARE);
+
     if (RegOpenKeyExW (HKEY_CURRENT_USER, szThemeManager,
                        0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS) 
     {
@@ -340,14 +349,14 @@ static void UXTHEME_RestoreSystemMetrics(void)
             for (i = 0; i < NUM_SYS_COLORS; i++)
             {
                 DWORD type;
-                char colorStr[13];
+                WCHAR colorStr[13];
                 DWORD count = sizeof(colorStr);
-            
-                if (RegQueryValueExA (colorKey, SysColorsNames[i], 0,
-                    &type, (LPBYTE) colorStr, &count) == ERROR_SUCCESS)
+
+                if (RegQueryValueExW(colorKey, SysColorsNames[i], 0, &type, (LPBYTE)colorStr,
+                                     &count) == ERROR_SUCCESS)
                 {
                     int r, g, b;
-                    if (sscanf (colorStr, "%d %d %d", &r, &g, &b) == 3)
+                    if (swscanf(colorStr, L"%d %d %d", &r, &g, &b) == 3)
                     {
                         sysColsIndices[sysColCount] = i;
                         sysCols[sysColCount] = RGB(r, g, b);
@@ -402,6 +411,8 @@ static void UXTHEME_RestoreSystemMetrics(void)
       
         RegCloseKey (hKey);
     }
+
+    SetThreadDpiAwarenessContext(old_context);
 }
 
 /* Make system settings persistent, so they're in effect even w/o uxtheme 
@@ -507,6 +518,18 @@ void UXTHEME_InitSystem(HINSTANCE hInst)
     atDialogThemeEnabled = GlobalAddAtomW(L"ux_dialogtheme");
 
     UXTHEME_LoadTheme();
+    ThemeHooksInstall();
+}
+
+void UXTHEME_UninitSystem(void)
+{
+    ThemeHooksRemove();
+    MSSTYLES_SetActiveTheme(NULL, FALSE);
+
+    GlobalDeleteAtom(atWindowTheme);
+    GlobalDeleteAtom(atSubAppName);
+    GlobalDeleteAtom(atSubIdList);
+    GlobalDeleteAtom(atDialogThemeEnabled);
 }
 
 /***********************************************************************
@@ -548,20 +571,14 @@ BOOL WINAPI IsCompositionActive(void)
 HRESULT WINAPI EnableTheming(BOOL fEnable)
 {
     HKEY hKey;
-    WCHAR szEnabled[] = L"0";
 
     TRACE("(%d)\n", fEnable);
 
-    if(fEnable != bThemeActive) {
-        if(fEnable) 
-            UXTHEME_BackupSystemMetrics();
-        else
-            UXTHEME_RestoreSystemMetrics();
-        UXTHEME_SaveSystemMetrics ();
+    if (bThemeActive && !fEnable)
+    {
         bThemeActive = fEnable;
-        if(bThemeActive) szEnabled[0] = '1';
         if(!RegOpenKeyW(HKEY_CURRENT_USER, szThemeManager, &hKey)) {
-            RegSetValueExW(hKey, L"ThemeActive", 0, REG_SZ, (BYTE*)szEnabled, sizeof(WCHAR));
+            RegSetValueExW(hKey, L"ThemeActive", 0, REG_SZ, (BYTE *)L"0", 2 * sizeof(WCHAR));
             RegCloseKey(hKey);
         }
 	UXTHEME_broadcast_msg (NULL, WM_THEMECHANGED);
@@ -603,10 +620,7 @@ static LPWSTR UXTHEME_GetWindowProperty(HWND hwnd, ATOM aProp, LPWSTR pszBuffer,
     return NULL;
 }
 
-/***********************************************************************
- *      OpenThemeDataEx                                     (UXTHEME.61)
- */
-HTHEME WINAPI OpenThemeDataEx(HWND hwnd, LPCWSTR pszClassList, DWORD flags)
+static HTHEME open_theme_data(HWND hwnd, LPCWSTR pszClassList, DWORD flags, UINT dpi)
 {
     WCHAR szAppBuff[256];
     WCHAR szClassBuff[256];
@@ -633,14 +647,42 @@ HTHEME WINAPI OpenThemeDataEx(HWND hwnd, LPCWSTR pszClassList, DWORD flags)
             pszUseClassList = pszClassList;
 
         if (pszUseClassList)
-            hTheme = MSSTYLES_OpenThemeClass(pszAppName, pszUseClassList);
+            hTheme = MSSTYLES_OpenThemeClass(pszAppName, pszUseClassList, dpi);
+
+        /* Fall back to default class if the specified subclass is not found */
+        if (!hTheme)
+            hTheme = MSSTYLES_OpenThemeClass(NULL, pszUseClassList, dpi);
     }
     if(IsWindow(hwnd))
         SetPropW(hwnd, (LPCWSTR)MAKEINTATOM(atWindowTheme), hTheme);
     TRACE(" = %p\n", hTheme);
+
+    SetLastError(hTheme ? ERROR_SUCCESS : E_PROP_ID_UNSUPPORTED);
     return hTheme;
 }
 
+/***********************************************************************
+ *      OpenThemeDataEx                                     (UXTHEME.61)
+ */
+HTHEME WINAPI OpenThemeDataEx(HWND hwnd, LPCWSTR pszClassList, DWORD flags)
+{
+    UINT dpi;
+
+    dpi = GetDpiForWindow(hwnd);
+    if (!dpi)
+        dpi = 96;
+
+    return open_theme_data(hwnd, pszClassList, flags, dpi);
+}
+
+/***********************************************************************
+ *      OpenThemeDataForDpi                                 (UXTHEME.@)
+ */
+HTHEME WINAPI OpenThemeDataForDpi(HWND hwnd, LPCWSTR class_list, UINT dpi)
+{
+    return open_theme_data(hwnd, class_list, 0, dpi);
+}
+
 /***********************************************************************
  *      OpenThemeData                                       (UXTHEME.@)
  */
@@ -663,6 +705,13 @@ HTHEME WINAPI OpenThemeData(HWND hwnd, LPCWSTR classlist)
 HTHEME WINAPI GetWindowTheme(HWND hwnd)
 {
     TRACE("(%p)\n", hwnd);
+
+    if (!hwnd)
+    {
+        SetLastError(E_HANDLE);
+        return NULL;
+    }
+
     return GetPropW(hwnd, (LPCWSTR)MAKEINTATOM(atWindowTheme));
 }
 
@@ -677,11 +726,15 @@ HRESULT WINAPI SetWindowTheme(HWND hwnd, LPCWSTR pszSubAppName,
     HRESULT hr;
     TRACE("(%p,%s,%s)\n", hwnd, debugstr_w(pszSubAppName),
           debugstr_w(pszSubIdList));
+
+    if (!hwnd)
+        return E_HANDLE;
+
     hr = UXTHEME_SetWindowProperty(hwnd, atSubAppName, pszSubAppName);
     if(SUCCEEDED(hr))
         hr = UXTHEME_SetWindowProperty(hwnd, atSubIdList, pszSubIdList);
     if(SUCCEEDED(hr))
-	UXTHEME_broadcast_msg (hwnd, WM_THEMECHANGED);
+        SendMessageW(hwnd, WM_THEMECHANGED, 0, 0);
     return hr;
 }
 
@@ -1193,3 +1246,19 @@ HRESULT WINAPI CheckThemeSignature(LPCWSTR pszThemeFileName)
     MSSTYLES_CloseThemeFile(pt);
     return S_OK;
 }
+
+BOOL WINAPI ThemeHooksInstall(void)
+{
+    struct user_api_hook hooks;
+
+    hooks.pDefDlgProc = UXTHEME_DefDlgProc;
+    hooks.pScrollBarDraw = UXTHEME_ScrollBarDraw;
+    hooks.pScrollBarWndProc = UXTHEME_ScrollbarWndProc;
+    return RegisterUserApiHook(&hooks, &user_api);
+}
+
+BOOL WINAPI ThemeHooksRemove(void)
+{
+    UnregisterUserApiHook();
+    return TRUE;
+}
