@@ -1291,9 +1291,9 @@ static inline BOOL get_surface_rect( const RECT *visible_rect, RECT *surface_rec
 /***********************************************************************
  *           ANDROID_WindowPosChanging
  */
-void CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_flags,
-                                     const RECT *window_rect, const RECT *client_rect, RECT *visible_rect,
-                                     struct window_surface **surface )
+BOOL CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_flags,
+                                      const RECT *window_rect, const RECT *client_rect, RECT *visible_rect,
+                                      struct window_surface **surface )
 {
     struct android_win_data *data = get_win_data( hwnd );
     RECT surface_rect;
@@ -1306,7 +1306,7 @@ void CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_fla
            hwnd, wine_dbgstr_rect(window_rect), wine_dbgstr_rect(client_rect),
            GetWindowLongW( hwnd, GWL_STYLE ), swp_flags );
 
-    if (!data && !(data = create_win_data( hwnd, window_rect, client_rect ))) return;
+    if (!data && !(data = create_win_data( hwnd, window_rect, client_rect ))) return TRUE;
 
     *visible_rect = *window_rect;
 
@@ -1339,6 +1339,7 @@ void CDECL ANDROID_WindowPosChanging( HWND hwnd, HWND insert_after, UINT swp_fla
 
 done:
     release_win_data( data );
+    return TRUE;
 }
 
 
@@ -1385,7 +1386,6 @@ void CDECL ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flag
  */
 UINT CDECL ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp )
 {
-    if (IsRectEmpty( rect )) return swp;
     if (!IsIconic( hwnd )) return swp;
     /* always hide icons off-screen */
     if (rect->left != -32000 || rect->top != -32000)
