@@ -1772,7 +1772,7 @@ static BOOL be_arm_is_jump(const void* insn, ADDRESS64* jumpee)
 
 static BOOL be_arm_insert_Xpoint(HANDLE hProcess, const struct be_process_io* pio,
                                  dbg_ctx_t *ctx, enum be_xpoint_type type,
-                                 void* addr, unsigned long* val, unsigned size)
+                                 void* addr, unsigned *val, unsigned size)
 {
     SIZE_T              sz;
 
@@ -1790,7 +1790,7 @@ static BOOL be_arm_insert_Xpoint(HANDLE hProcess, const struct be_process_io* pi
 
 static BOOL be_arm_remove_Xpoint(HANDLE hProcess, const struct be_process_io* pio,
                                  dbg_ctx_t *ctx, enum be_xpoint_type type,
-                                 void* addr, unsigned long val, unsigned size)
+                                 void* addr, unsigned val, unsigned size)
 {
     SIZE_T              sz;
 
@@ -1851,10 +1851,9 @@ static BOOL be_arm_fetch_integer(const struct dbg_lvalue* lvalue, unsigned size,
     return TRUE;
 }
 
-static BOOL be_arm_fetch_float(const struct dbg_lvalue* lvalue, unsigned size,
-                               long double* ret)
+static BOOL be_arm_fetch_float(const struct dbg_lvalue* lvalue, unsigned size, double *ret)
 {
-    char        tmp[sizeof(long double)];
+    char tmp[sizeof(double)];
 
     /* FIXME: this assumes that debuggee and debugger use the same
      * representation for reals
@@ -1863,7 +1862,6 @@ static BOOL be_arm_fetch_float(const struct dbg_lvalue* lvalue, unsigned size,
 
     if (size == sizeof(float)) *ret = *(float*)tmp;
     else if (size == sizeof(double)) *ret = *(double*)tmp;
-    else if (size == sizeof(long double)) *ret = *(long double*)tmp;
     else return FALSE;
 
     return TRUE;
