@@ -21,26 +21,13 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
 #include <time.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
-#ifdef HAVE_SYS_MMAN_H
-#include <sys/mman.h>
-#endif
-#include <fcntl.h>
 
+#include "../tools.h"
 #include "windef.h"
 #include "winbase.h"
 #include "winedump.h"
@@ -683,7 +670,7 @@ void    dump_stabs(const void* pv_stabs, unsigned szstabs, const char* stabstr,
      * where the stab is continued over multiple lines.
      */
     stabbufflen = 65536;
-    stabbuff = malloc(stabbufflen);
+    stabbuff = xmalloc(stabbufflen);
 
     stabbuff[0] = '\0';
 
@@ -707,7 +694,7 @@ void    dump_stabs(const void* pv_stabs, unsigned szstabs, const char* stabstr,
             if (strlen(stabbuff) + len > stabbufflen)
             {
                 stabbufflen += 65536;
-                stabbuff = realloc(stabbuff, stabbufflen);
+                stabbuff = xrealloc(stabbuff, stabbufflen);
             }
             strcat(stabbuff, ptr);
             continue;
