@@ -41,8 +41,7 @@ extern int no_preprocess;
 extern int utf8_input;
 extern int check_utf8;
 
-extern char *input_name;
-
+extern const char *input_name;
 extern const char *nlsdirs[];
 
 extern int line_number;
